==> spread-5.2.0.24-py3-none-any.whl: spread_core/bam/mercury/providers.py <==
import logging
import datetime

import spread_core.bam as bam
import spread_core.mqtt as mqtt
import spread_mercury as mercury
from spread_core.errors.project_errors import ClassifierError, InvalidDevice
from spread_core.tools.signature import CommandSignature
from spread_mercury import responses
from spread_mercury.errors import TimeOutError, AccessError


class MercuryElectricMeter(bam.Provider):
    def __init__(self, data):
        super(MercuryElectricMeter, self).__init__(data)
        self.address = data['attributes']['address']
        self.passwords = dict()
        for i in range(1, 4):
            if 'password{}'.format(i) in data['attributes']:
                self.passwords[i] = data['attributes']['password{}'.format(i)]

    def on_request_answer(self, funit_type, response, sig=None):
        if isinstance(response, (responses.MercuryErrorResponse, InvalidDevice)):
            if funit_type == Validity:
                val = False
            else:
                val = None
        elif isinstance(response.value, dict) and funit_type in response.value:
            val = response.value[funit_type]
        else:
            val = response.value

        self.on_update(funit_type, val, sig=response.cmd.sig)

    def request(self, cmd: mercury.commands.MCommand, *args, **kwargs):
        try:
            if self.is_valid or isinstance(cmd, mercury.commands.CheckConnect):
                self._manager.send_interface.send(cmd, *args, **kwargs)
            else:
                raise InvalidDevice(cmd)
        except BaseException as ex:
            cmd.response = responses.MercuryErrorResponse(cmd, b'')

            if isinstance(ex, (TimeOutError, AccessError)):
                logging.error(ex)
            elif isinstance(ex, InvalidDevice):
                cmd.response.value = False
                logging.error(ex)
            else:
                logging.exception(ex)

        if cmd.response:
            for funit_type in cmd.funit_type:
                self.on_request_answer(funit_type, cmd.response)

    def on_survey(self):
        yield bam.MultiCommand(self.check_valid)
        yield bam.MultiCommand(self.read_voltage)
        yield bam.MultiCommand(self.read_amperage)
        yield bam.MultiCommand(self.read_angle)
        yield bam.MultiCommand(self.read_power_active)
        yield bam.MultiCommand(self.read_power_reactive)
        yield bam.MultiCommand(self.read_power_total)
        yield bam.MultiCommand(self.read_power_rate)
        yield bam.MultiCommand(self.read_frequency)

    def on_update(self, funit_type, value, sig=None):
        if funit_type == Validity:
            if value != self.values.get(funit_type, None):
                self.set_valid(self.VALID if value else self.INVALID)
                if value:
                    self.get_info()

        super(MercuryElectricMeter, self).on_update(funit_type, value, sig)

    def get_bindings(self):
        self.on_update(Info, dict(address=self.address, name=self.name))

    def get_info(self):
        # self._manager.add_info(self, mercury.commands.Features(addr=self.address))
        self._manager.add_info(self, mercury.commands.ReadLoadState(addr=self.address))
        self._manager.add_info(self, mercury.commands.SerialCommand(addr=self.address))
        self._manager.add_info(self, mercury.commands.CurrentDateTime(addr=self.address))
        self._manager.add_info(self, mercury.commands.TransformRate(addr=self.address))

    def _read_aux(self, funit_type, aux, sig=None, **kwargs):
        if self.is_valid:
            assert issubclass(aux, mercury.commands.AUX)
            result = None
            try:
                if aux._min_phase == aux._max_phase:
                    result = self.send(aux(addr=self.address, **kwargs)).value
                else:
                    result = {
                        'total' if phase == 0 else f'phase{phase}': self.send(
                            aux(addr=self.address, phase=phase, **kwargs)
                        ).value for phase in range(aux._min_phase, aux._max_phase+1)
                    }
            except BaseException as ex:
                logging.error(ex)
            finally:
                self.on_update(funit_type, result, sig=sig)

    def get_stored_energy(self, period, tariff=0, month=1, sig=None):
        topic = mqtt.spread.topic.Reply(self._manager.project_id, sig.session_id)

        try:
            if self.is_valid:
                if not str(period).isdigit():
                    cmd = mercury.commands.StoredEnergyOfDate(
                        self.address,
                        date=datetime.date.fromisoformat(period),
                        tariff=tariff,
                        section=mercury.commands.StoredEnergyOfDate.DAY
                    )
                else:
                    cmd = mercury.commands.StoredEnergy(self.address, period, tariff, month)
                r = self.send(cmd)
                r = r.cmd.res_pack(None)
                packet = mqtt.spread.variable.Variable(r.pop('value'), key=sig.key, params=r)
            else:
                raise
        except BaseException as ex:
            packet = mqtt.spread.variable.Variable(None)

        self.publish(topic, packet)

    def check_valid(self): self.request(mercury.commands.CheckConnect(self.address))

    def _read_power(self, funit_type, power_id, sig=None): self._read_aux(funit_type, mercury.commands.AUXPower, power_id=power_id, sig=sig)

    def read_voltage(self, sig=None): self._read_aux(Voltage, mercury.commands.AUXVoltage, sig=sig)
    def read_amperage(self, sig=None): self._read_aux(Amperage, mercury.commands.AUXAmperage, sig=sig)
    def read_angle(self, sig=None): self._read_aux(Angle, mercury.commands.AUXAngle, sig=sig)
    def read_frequency(self, sig=None): self._read_aux(Frequency, mercury.commands.AUXFrequency, sig=sig)
    def read_power_rate(self, sig=None): self._read_aux(PowerRate, mercury.commands.AUXPowerRate, sig=sig)
    def read_power_active(self, sig=None): self._read_power(PowerActive, power_id=0, sig=sig)
    def read_power_reactive(self, sig=None): self._read_power(PowerReactive, power_id=1, sig=sig)
    def read_power_total(self, sig=None): self._read_power(PowerTotal, power_id=2, sig=sig)


class MEMFunit(bam.Funit):
    set_cmd = None
    get_cmd = None
    set_func = None
    get_func = None

    @property
    def entity(self) -> MercuryElectricMeter:
        return super(MEMFunit, self).entity

    def set(self):
        if self.set_cmd:
            cmd = self.set_cmd(self.entity.address, self.set_value, **self.params)
            if cmd.access_level == 2:
                if self._variable.params.get('password', '') != self.entity.passwords[2]:
                    raise ClassifierError('Неверный пароль')
            self.entity.manager.add_command(self.entity, cmd)
        elif self.set_func:
            cmd = bam.MultiCommand(getattr(self.entity, self.set_func.__name__), **self.params)
            self.entity.manager.add_command(self.entity, cmd)

    def get(self):
        if self.get_cmd:
            cmd = self.get_cmd(self.entity.address, **self.params)
            if cmd.access_level == 2:
                if self._variable.params.get('password', '') != self.entity.passwords[2]:
                    raise ClassifierError('Неверный пароль')
            cmd.sig = CommandSignature(self.funit_type, session_id=self._variable.session_id, key=self._variable.key)
            self.entity.manager.add_command(self.entity, cmd)
        elif self.get_func:
            cmd = bam.MultiCommand(getattr(self.entity, self.get_func.__name__), **self.params)
            cmd.set_signature(self.funit_type, session_id=self._variable.session_id, key=self._variable.key)
            self.entity.manager.add_command(self.entity, cmd)


MercuryElectricMeter.funit_class = MEMFunit


class Validity(MEMFunit):
    get_cmd = mercury.commands.CheckConnect


class Info(MEMFunit):
    get_func = MercuryElectricMeter.read_power_active


class PowerActive(MEMFunit):
    get_func = MercuryElectricMeter.read_power_active


class PowerReactive(MEMFunit):
    get_func = MercuryElectricMeter.read_power_reactive


class PowerTotal(MEMFunit):
    get_func = MercuryElectricMeter.read_power_total


class Voltage(MEMFunit):
    get_func = MercuryElectricMeter.read_voltage


class Amperage(MEMFunit):
    get_func = MercuryElectricMeter.read_amperage


class Angle(MEMFunit):
    get_func = MercuryElectricMeter.read_angle


class Frequency(MEMFunit):
    get_func = MercuryElectricMeter.read_frequency


class PowerRate(MEMFunit):
    get_func = MercuryElectricMeter.read_power_rate


class StoredEnergy(MEMFunit):
    get_func = MercuryElectricMeter.get_stored_energy
    @property
    def params(self) -> dict:
        return dict(period=self._variable.params['period'],
                    tariff=self._variable.params['tariff'],
                    month=self._variable.params.get('month', 0))


class Temperature(MEMFunit):
    get_cmd = mercury.commands.AUXTemp


class SerialDate(MEMFunit):
    get_cmd = mercury.commands.SerialCommand


class CurrentDateTime(MEMFunit):
    get_cmd = mercury.commands.CurrentDateTime


class OpenCloseTime(MEMFunit):
    get_cmd = mercury.commands.OpenCloseTime


class Version(MEMFunit):
    get_cmd = mercury.commands.Version


class RateVoltage(MEMFunit):
    set_cmd = mercury.commands.SetTransformRate
    get_cmd = mercury.commands.TransformRate


class RateAmperage(MEMFunit):
    set_cmd = mercury.commands.SetTransformRate
    get_cmd = mercury.commands.TransformRate


class PulseMode(MEMFunit):
    set_cmd = mercury.commands.SetPulseMode
    get_cmd = mercury.commands.ReadLoadState


class Status(MEMFunit):
    get_cmd = mercury.commands.ReadLoadState


class LoadOn(MEMFunit):
    set_cmd = mercury.commands.SetLoad
    get_cmd = mercury.commands.ReadLoadState

==> spread-5.3.0.5-py3-none-any.whl: spread_core/bam/mercury/providers.py <==
import logging
import datetime

import spread_core.bam as bam
import spread_core.mqtt as mqtt
import spread_mercury as mercury
from spread_core.errors.project_errors import ClassifierError, InvalidDevice
from spread_core.tools.signature import CommandSignature
from spread_mercury import responses
from spread_mercury.errors import TimeOutError, AccessError


class MercuryElectricMeter(bam.Provider):
    def __init__(self, data):
        super(MercuryElectricMeter, self).__init__(data)
        self.address = data['attributes']['address']
        self.passwords = dict()
        for i in range(1, 4):
            if 'password{}'.format(i) in data['attributes']:
                self.passwords[i] = data['attributes']['password{}'.format(i)]

    def on_request_answer(self, funit_type, response, sig=None, **kwargs):
        if isinstance(response, (responses.MercuryErrorResponse, InvalidDevice)):
            if funit_type == Validity:
                val = False
            else:
                val = None
        elif isinstance(response.value, dict) and funit_type in response.value:
            val = response.value[funit_type]
        else:
            val = response.value

        self.on_update(funit_type, val, sig=response.cmd.sig)

    def request(self, cmd: mercury.commands.MCommand, *args, **kwargs):
        try:
            if self.is_valid or isinstance(cmd, mercury.commands.CheckConnect):
                self._manager.send_interface.send(cmd, *args, **kwargs)
            else:
                raise InvalidDevice(cmd)
        except BaseException as ex:
            cmd.response = responses.MercuryErrorResponse(cmd, b'')

            if isinstance(ex, (TimeOutError, AccessError)):
                logging.error(ex)
            elif isinstance(ex, InvalidDevice):
                cmd.response.value = False
                logging.error(ex)
            else:
                logging.exception(ex)

        if cmd.response:
            for funit_type in cmd.funit_type:
                self.on_request_answer(funit_type, cmd.response)

    def on_survey(self):
        yield bam.MultiCommand(self.check_valid)
        yield bam.MultiCommand(self.read_voltage)
        yield bam.MultiCommand(self.read_amperage)
        yield bam.MultiCommand(self.read_angle)
        yield bam.MultiCommand(self.read_power_active)
        yield bam.MultiCommand(self.read_power_reactive)
        yield bam.MultiCommand(self.read_power_total)
        yield bam.MultiCommand(self.read_power_rate)
        yield bam.MultiCommand(self.read_frequency)

    def on_update(self, funit_type, value, sig=None, **kwargs):
        if funit_type == Validity:
            if value != self.values.get(funit_type, None):
                self.set_valid(self.VALID if value else self.INVALID)
                if value:
                    self.get_info()

        super(MercuryElectricMeter, self).on_update(funit_type, value, sig)

    def get_bindings(self):
        self.on_update(Info, dict(address=self.address, name=self.name))

    def get_info(self):
        # self._manager.add_info(self, mercury.commands.Features(addr=self.address))
        self._manager.add_info(self, mercury.commands.ReadLoadState(addr=self.address))
        self._manager.add_info(self, mercury.commands.SerialCommand(addr=self.address))
        self._manager.add_info(self, mercury.commands.CurrentDateTime(addr=self.address))
        self._manager.add_info(self, mercury.commands.TransformRate(addr=self.address))

    def _read_aux(self, funit_type, aux, sig=None, **kwargs):
        if self.is_valid:
            assert issubclass(aux, mercury.commands.AUX)
            result = None
            try:
                if aux._min_phase == aux._max_phase:
                    result = self._manager.send_interface.send(aux(addr=self.address, **kwargs)).value
                else:
                    result = {
                        'total' if phase == 0 else f'phase{phase}': self._manager.send_interface.send(
                            aux(addr=self.address, phase=phase, **kwargs)
                        ).value for phase in range(aux._min_phase, aux._max_phase+1)
                    }
            except BaseException as ex:
                logging.error(ex)
            finally:
                self.on_update(funit_type, result, sig=sig)

    def get_stored_energy(self, period, tariff=0, month=1, sig=None):
        topic = mqtt.spread.topic.Reply(self._manager.project_id, sig.session_id)

        try:
            if self.is_valid:
                if not str(period).isdigit():
                    cmd = mercury.commands.StoredEnergyOfDate(
                        self.address,
                        date=datetime.date.fromisoformat(period),
                        tariff=tariff,
                        section=mercury.commands.StoredEnergyOfDate.DAY
                    )
                else:
                    cmd = mercury.commands.StoredEnergy(self.address, period, tariff, month)
                r = self._manager.send_interface.send(cmd)
                r = r.cmd.res_pack(None)
                packet = mqtt.spread.variable.Variable(r.pop('value'), key=sig.key, params=r)
            else:
                raise
        except BaseException as ex:
            packet = mqtt.spread.variable.Variable(None)

        self.publish(topic, packet)

    def check_valid(self): self.request(mercury.commands.CheckConnect(self.address))

    def _read_power(self, funit_type, power_id, sig=None): self._read_aux(funit_type, mercury.commands.AUXPower, power_id=power_id, sig=sig)

    def read_voltage(self, sig=None): self._read_aux(Voltage, mercury.commands.AUXVoltage, sig=sig)
    def read_amperage(self, sig=None): self._read_aux(Amperage, mercury.commands.AUXAmperage, sig=sig)
    def read_angle(self, sig=None): self._read_aux(Angle, mercury.commands.AUXAngle, sig=sig)
    def read_frequency(self, sig=None): self._read_aux(Frequency, mercury.commands.AUXFrequency, sig=sig)
    def read_power_rate(self, sig=None): self._read_aux(PowerRate, mercury.commands.AUXPowerRate, sig=sig)
    def read_power_active(self, sig=None): self._read_power(PowerActive, power_id=0, sig=sig)
    def read_power_reactive(self, sig=None): self._read_power(PowerReactive, power_id=1, sig=sig)
    def read_power_total(self, sig=None): self._read_power(PowerTotal, power_id=2, sig=sig)


class MEMFunit(bam.Funit):
    set_cmd = None
    get_cmd = None
    set_func = None
    get_func = None

    @property
    def entity(self) -> MercuryElectricMeter:
        return super(MEMFunit, self).entity

    def set(self):
        if self.set_cmd:
            cmd = self.set_cmd(self.entity.address, self.set_value, **self.params)
            if cmd.access_level == 2:
                if self._variable.params.get('password', '') != self.entity.passwords[2]:
                    raise ClassifierError('Неверный пароль')
            self.entity.manager.add_command(self.entity, cmd)
        elif self.set_func:
            cmd = bam.MultiCommand(getattr(self.entity, self.set_func.__name__), **self.params)
            self.entity.manager.add_command(self.entity, cmd)

    def get(self):
        if self.get_cmd:
            cmd = self.get_cmd(self.entity.address, **self.params)
            if cmd.access_level == 2:
                if self._variable.params.get('password', '') != self.entity.passwords[2]:
                    raise ClassifierError('Неверный пароль')
            cmd.sig = CommandSignature(self.funit_type, session_id=self._variable.session_id, key=self._variable.key)
            self.entity.manager.add_command(self.entity, cmd)
        elif self.get_func:
            cmd = bam.MultiCommand(getattr(self.entity, self.get_func.__name__), **self.params)
            cmd.set_signature(self.funit_type, session_id=self._variable.session_id, key=self._variable.key)
            self.entity.manager.add_command(self.entity, cmd)


MercuryElectricMeter.funit_class = MEMFunit


class Validity(MEMFunit):
    get_cmd = mercury.commands.CheckConnect


class Info(MEMFunit):
    get_func = MercuryElectricMeter.read_power_active


class PowerActive(MEMFunit):
    get_func = MercuryElectricMeter.read_power_active


class PowerReactive(MEMFunit):
    get_func = MercuryElectricMeter.read_power_reactive


class PowerTotal(MEMFunit):
    get_func = MercuryElectricMeter.read_power_total


class Voltage(MEMFunit):
    get_func = MercuryElectricMeter.read_voltage


class Amperage(MEMFunit):
    get_func = MercuryElectricMeter.read_amperage


class Angle(MEMFunit):
    get_func = MercuryElectricMeter.read_angle


class Frequency(MEMFunit):
    get_func = MercuryElectricMeter.read_frequency


class PowerRate(MEMFunit):
    get_func = MercuryElectricMeter.read_power_rate


class StoredEnergy(MEMFunit):
    get_func = MercuryElectricMeter.get_stored_energy
    @property
    def params(self) -> dict:
        return dict(period=self._variable.params['period'],
                    tariff=self._variable.params['tariff'],
                    month=self._variable.params.get('month', 0))


class Temperature(MEMFunit):
    get_cmd = mercury.commands.AUXTemp


class SerialDate(MEMFunit):
    get_cmd = mercury.commands.SerialCommand


class CurrentDateTime(MEMFunit):
    get_cmd = mercury.commands.CurrentDateTime


class OpenCloseTime(MEMFunit):
    get_cmd = mercury.commands.OpenCloseTime


class Version(MEMFunit):
    get_cmd = mercury.commands.Version


class RateVoltage(MEMFunit):
    set_cmd = mercury.commands.SetTransformRate
    get_cmd = mercury.commands.TransformRate


class RateAmperage(MEMFunit):
    set_cmd = mercury.commands.SetTransformRate
    get_cmd = mercury.commands.TransformRate


class PulseMode(MEMFunit):
    set_cmd = mercury.commands.SetPulseMode
    get_cmd = mercury.commands.ReadLoadState


class Status(MEMFunit):
    get_cmd = mercury.commands.ReadLoadState


class LoadOn(MEMFunit):
    set_cmd = mercury.commands.SetLoad
    get_cmd = mercury.commands.ReadLoadState
